#!/usr/bin/perl

# "readable.pl", a script for use with the Forum News Gateway
# copyright 1995 by Swarthmore College

# Convert a possibly ugly tab-delimited stream on standard input
# into a neatly-justified, space-delimited, human-readable stream
# on standard out. Whether the output is actually readable is a
# matter of opinion. :-) It helps to have a wide screen.

# This grabs the entire input stream into memory (to look at it
# twice), so it's not suitable for huge inputs. Large, yes, huge, no.

# --

# Gulp all the input at once.
@lines = <STDIN>;

# Go through the input once to count the columns and decide on the column widths.
@widths = ();				# width of each column
foreach $line (@lines) {
	chop ($line);
	@fields = split (/\t/, $line);
	
	# If there are not enough entries in @widths, create new ones.
	while (@fields > @widths) {
		push (@widths, 0);
	}
	
	# Increase the width of any column that is too narrow for its contents.
	for ($i = 0; $i < @fields; ++$i) {
		$length = length ($fields [$i]);
		$widths [$i] = $length if ($widths [$i] < $length);
	}
}

# Go through the input again to format it and print it.
foreach $line (@lines) {
	@fields = split (/\t/, $line);
	
	$out = "";
	for ($i = 0; $i < @fields; ++$i) {
		$width = 1 + $widths [$i];
		$out .= sprintf ("%-${width}s", $fields [$i]);
	}
	print $out, "\n";
}
