# "config.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College


# ----- news servers

# List the NNTP servers that will be accessible by users.
# In the form "host.domain.edu:port", but the port can be omitted if it
# is 119 (the standard NNTP port).  The first server in the list
# is considered the default server, and any URL with no server specified
# in it (this includes news:group.name and news:[msgid] URLs) will be
# assumed to refer to the default server. Servers which do not exist
# (maybe you mistyped) are noticed on startup and ignored; if the first
# one is ignored then the next one becomes the default, and so on.

@NNTPServers = ("forum.swarthmore.edu","larch.cc.swarthmore.edu");

# You can give the news servers more readable names for the benefit of
# the non-nerd user.
# For each entry in @NNTPServers, write an alias in @NNTPServerAliases.
# If the list of aliases is empty, users see the bare machine names
# from @NNTPServers.

@NNTPServerAliases = ();

# Example:
# @NNTPServerAliases = ("Geometry Forum Server","Swarthmore College Server");

# How long should an NNTP connection be allowed to linger without being
# closed and re-opened? A connection is considered "stale" if it is older
# than $NNTPConnectionSeconds seconds. Stale connections are closed, and
# their associated data is flushed to disk, protecting it in case of a crash.

# There's at least one NNTP server out there which will not tell the gateway
# about newly-arrived articles unless the gateway closes and re-opens the
# NNTP connection. If you find that news home pages insist that all subscribed
# groups are empty when you know that new articles have arrived, this may be
# the problem. Set $NNTPConnectionSeconds to some small value, such as fifteen
# minutes (15*60 seconds).

$NNTPConnectionSeconds = 2*60*60;


# ----- bug workarounds

# -- Fetch articles by number.

# The problem:
# Some defective NNTP servers cannot fetch articles by article ID. Painful
# but true. If you suffer from one of these mush-brain mutants, set
# $FetchArticlesByNumber = 1 to fetch articles by article number instead.
# The setting affects all news servers.

# The symptoms:
# If, when trying to read articles, you get systematic "Your request could
# not be satisfied" messages, and the message mentions that the news server
# returned code 430, then this is probably the difficulty. For some news
# servers, you may get such errors only in certain newsgroup hierarchies.

# It is an option because I fear that somewhere there is another broken
# server which doesn't understand article numbers. Paranoia? You tell me.

$FetchArticlesByNumber = 0;


# -- Turn off SIGPIPE handling.

# The problem:
# The gateway normally relies on a SIGPIPE signal to tell it when
# the client web browser has closed the connection, breaking the
# gateway's output pipe. Unfortunately, on some systems the gateway
# is unable to return from handling a SIGPIPE signal, and crashes
# instead. The problem seems to be a bug or incompatibility in either
# perl or the operating system, I don't know which.

# The symptoms:
# You may be suffering this problem if, whenever you interrupt your
# browser while it is loading a gateway page, the gateway crashes
# with the message
# 	"Probably, the client closed the network connection early. (1)"
# The message is nothing to worry about; it should be common. Worry
# about the crashes.

# Set $IgnoreSIGPIPE = 1 if you suspect this problem. Leave it alone
# otherwise; I fear that ignoring SIGPIPE when you don't have to
# could cause problems on some systems.

$IgnoreSIGPIPE = 0;


# ----- working files

# The following variables relate to where various data files, such as
# newsrc's, killfiles, threadDB's and allgroups files, are stored.

# By default, all the data files are stored relative to the current directory.
# NOTE: If you change this to something like
#	$WorkingDirectory = "/some/directory";
# then don't forget to remove the chop() line just below!
# Also, if you change $WorkingDirectory, make sure that the .html files
# currently in "./HTMLFiles" can still be found. You can do that by either
# moving the HTMLFiles directory into the new working directory, or by
# making sure that $HTMLFilesRoot, below, still points to the right place.
$WorkingDirectory = `pwd`;
chop($WorkingDirectory);

$PublicDataFilesRoot = $WorkingDirectory."/PublicFiles";

# The directory which contains html files to be served.
# Most pages served are generated on the fly, but some are
# looked up in this directory.

$HTMLFilesRoot = $WorkingDirectory."/HTMLFiles";

# Newsrc's and killfiles go in here, in a subdirectory with the same name
# as the user.
$UserFilesDirectoryName = $WorkingDirectory."/UserFiles";

$ThreadDBFilesDirectoryName = $PublicDataFilesRoot."/ThreadDBs";
$AllGroupsFilesDirectoryName = $PublicDataFilesRoot."/AllGroups";


# ----- personal information restrictions

# User names must be at least this long, in characters.

$MinUserNameLength = 2;

# User passwords must be at least this long, in characters.

$MinPasswordLength = 4;

# User e-mail addresses must match this regular expression to be
# considered valid. The default pattern is simpleminded; it requires only
# that the putative e-mail address contain a "@" character. If you set the
# pattern to "", then users are not required to fill in their e-mail
# addresses, even to post articles (any bad effects of that are your
# responsibility :-).

$EmailValidityPattern = "\@";

# Here's a slightly more restrictive pattern, which requires characters
# before and after the @:

# $EmailValidityPattern = "[^\@]+\@[^\@]+";

1;
