# "globals.pl" in the Forum News Gateway
# copyright 1994,1995 by Swarthmore College

# This file contains routines that initialize and clean up the global
# variables.  Includes a signal handler that performs all necessary cleanup.

# InitGlobalVars()
# Initialize global variables.

sub InitGlobalVars {
	srand;								# set the random number seed
	
#	$/ = "\015\012";					# causes trouble for perl5
	$ExceptionCode = "";
	$ExceptionText = "";
	
	# Exception codes for non-NNTP errors. These must all be less than zero!
	$ERR_GENERIC = -1;
	$ERR_CLIENT_DIED = -2;
	$ERR_CANT_OPEN_FILE = -3;			# for &SendFileToClient in "httpd.pl"
	$ERR_COULDNT_MKDIR = -4;
	$ERR_NO_NAME_ENTERED = -5;
	$ERR_BAD_OVERVIEW_DATA = -6;
	
	local($OldOutFile) = select(STDERR); $| = 1; select($OldOutFile);
	
	&MakeDirectory($PublicDataFilesRoot,0700,"Couldn't make directory for public data files: $!");
	&MakeDirectory($AllGroupsFilesDirectoryName,0700,"Couldn't make directory for allgroups files: $!");
	
	&MakeDirectory($UserFilesDirectoryName,0700,"Couldn't make directory for user-specific files: $!");

	$CurrentUser = "";
	%Newsrc = ();
	%KillFile = ();
	&SetCurrentUser("");

	
	&MakeDirectory($ThreadDBFilesDirectoryName,0700,"Couldn't make directory for ThreadDB files: $!");
	
	$CurrentGroup = "";
	$CurrentGroupLowWaterMark = 0;
	$CurrentGroupHighWaterMark = 0;
	$CurrentGroupThreadDBLoaded = 0;
	%ThreadDB = ();
	&SetCurrentGroup("",0);
	
	$CurrentNNTPServer = "";
	$CurrentNNTPServerAllGroupsLoaded = 0;
	%AllGroups = ();
	
	# This is a hack so that the SIGPIPE signal handler can tell whether
	# the pipe broken signal was raised because the web client, on the CLIENT
	# file handle, broke the connection, or because the NNTP connection,
	# on the NNTPSERVER file handle, was broken. We set $NNTPCommandInProgress = 1
	# when about to send to NNTPSERVER, and otherwise leave it 0.
	# See routines PipeSigHandler and SendNNTPCommand.
	$NNTPCommandInProgress = 0;
	
	%SubscribedGroups = ();
	@SortedSubscribedGroups = ();
	
	# Set up the news server info.
	@CanonicalNNTPServers = ();
	%NNTPServerAliases = ();
	$NNTPConnectionOpenedAt = 0;		# will be time() when the NNTP connection was opened
	
	local($Server);
	local($CanonicalizedServer);
	local($ServerAlias);
	foreach $Server (@NNTPServers) {
		$ServerAlias = shift(@NNTPServerAliases);		# possibly "", which is OK
#		print STDERR "Initializing info on \"$Server\".\n";
		
		$CanonicalizedServer = &CanonicalizeServer($Server);
#		print STDERR "Canonicalized server name is \"$CanonicalizedServer\"\n";
		
		# If the news server doesn't exist, then &CanonicalizeServer will have returned ""
		# as the canonical server name. If that's what we got, complain.
		if ($CanonicalizedServer eq "") {
			print STDERR "Could not find news server \"$Server\"; ignoring it.\n";
		} else {
			push(@CanonicalNNTPServers,$Server);
			$NNTPServerAliases{$Server} = $ServerAlias;
		}
	}
	
	# Create or update the AllGroups files for each news server.
	foreach $Server (@CanonicalNNTPServers) {
		print STDERR "Checking for new groups on \"$Server\".\n";

		eval '&SetCurrentNNTPServer($Server,1)';
#		&SetCurrentNNTPServer($Server,1);
		
		if ($ExceptionCode) {
			# ignore errors - they will have shown up in the logfile
			$ExceptionCode = 0;
			$ExceptionText = "";
		}
	}
	
	&SetCurrentNNTPServer("",0);
	
	&OpenPassiveTCPSocket(LISTENER,$LocalPortNum) || die "Couldn't listen for connections: $!";
		
	if ($IgnoreSIGPIPE) {
		# Prevent crashes on buggy systems.
		$SIG{'PIPE'} = 'IGNORE';
	} else {
		# The usual case.
		$SIG{'PIPE'} = 'PipeSigHandler';
	}
	
	$SIG{'INT'} = 'IntHandler';
	$SIG{'TERM'} = 'IntHandler';
	$SIG{'QUIT'} = 'IntHandler';
	
	$SIG{'HUP'} = 'DefaultSigHandler';
	$SIG{'ILL'} = 'DefaultSigHandler';
	$SIG{'TRAP'} = 'DefaultSigHandler';
	$SIG{'ABRT'} = 'DefaultSigHandler';
	$SIG{'EMT'} = 'DefaultSigHandler';
	$SIG{'FPE'} = 'DefaultSigHandler';
	$SIG{'BUS'} = 'DefaultSigHandler';
	$SIG{'SEGV'} = 'DefaultSigHandler';
	$SIG{'SYS'} = 'DefaultSigHandler';
	$SIG{'ALRM'} = 'DefaultSigHandler';
	$SIG{'XCPU'} = 'DefaultSigHandler';
	$SIG{'XFSZ'} = 'DefaultSigHandler';
	$SIG{'VTALRM'} = 'DefaultSigHandler';
	$SIG{'PROF'} = 'DefaultSigHandler';
	$SIG{'LOST'} = 'DefaultSigHandler';
	$SIG{'USR1'} = 'DefaultSigHandler';
	$SIG{'USR2'} = 'DefaultSigHandler';
}

# ReconfigureGateway()
# Load the perl header file "reconfig.pl" to alter the gateway's configuration.
# Return whether the load succeeded.
# This routine is called once when the gateway starts up, to set
# the initial state of reconfigurable variables, and again later
# whenever the administrator calls for it.

sub ReconfigureGateway {
	eval `cat reconfig.pl`;
	
	if ($@ ne "" && $ExceptionCode == 0) {
		print STDERR "Perl error in \"reconfig.pl\": $@\n";
		return 0;
	}
	
	return 1;
}

# ServerAlias($NNTPServer)
# Return the alias of a given NNTP server. The alias may be a nice name
# for the user, taken from @NNTPServerAliases in "config.pl", or it may
# be only the server's bare name after all.

sub ServerAlias {
	local($NNTPServer) = @_;
	
	$Alias = $NNTPServerAliases{$NNTPServer};
	$Alias = $NNTPServer if ($Alias eq "");
	$Alias;
}

# LongServerAlias($NNTPServer)
# Return the long alias of a given NNTP server.
# If the server has an alias, then the long alias has the form
#	Server Alias (server.name)
# If the server has no alias, then the long alias is simply the machine name:
#	server.name

sub LongServerAlias {
	local($NNTPServer) = @_;
	
	local($Alias);
	
	$Alias = $NNTPServerAliases{$NNTPServer};
	if ($Alias eq "") {
		$Alias = $NNTPServer;
	} else {
		$Alias .= " (".$NNTPServer.")";
	}
	
	$Alias;
}

# CanonicalizeServer($NNTPServer)
# Convert an NNTP server name into canonical form, either "machine.name.wherever:4444"
# (for port 4444) or, if the port is omitted or is the default port of 119, simply
# "machine.name.wherever".
# If $NNTPServer is the name of a machine that does not exist, then the "canonical" form
# will be "". The caller must check.

sub CanonicalizeServer {
	local($NNTPServer) = @_;
	
	return "" if ($NNTPServer eq "");
	
	local($NNTPHost,$NNTPPort) = split(/\:/,$NNTPServer);
	
	# Canonicalize the hostname.
	($NNTPServer) = gethostbyname($NNTPHost);
	
	return "" if ($NNTPServer eq "");
	
	if ($NNTPPort =~ /^0*(\d+)/ && $1 ne "119") {
		$NNTPServer .= ":".$1;
	}
	
	return $NNTPServer;
}

# SetCurrentNNTPServer($NNTPServer,$LoadAllGroups)
# Change $CurrentNNTPServer to the given one. If $LoadAllGroups is 1,
# then load and update %AllGroups, the tree of newsgroups; otherwise leave it empty.

sub SetCurrentNNTPServer {
	local($NNTPServer,$LoadAllGroups) = @_;
	
#	print STDERR "Setting server to $NNTPServer $LoadAllGroups\n";
	# If the NNTP connection is stale, close it here and let it be reopened.
	# If we're changing NNTP servers anyway, then the connection will be closed
	# and reopened anyway, so there's no need to check here.
	if ($CurrentNNTPServer ne "" &&									# we have a connection
		$CurrentNNTPServer eq $NNTPServer &&						# we're not about to change it
		time - $NNTPConnectionOpenedAt > $NNTPConnectionSeconds)	# it's stale
	{
		if ($LogChanges) {
			print STDERR "News server connection is stale; flushing it.\n";
		}
		
		# Flush the group information to disk.
		if ($CurrentGroupThreadDBLoaded) {
			&MyDBMClose(*ThreadDB,&ThreadDBFilename($CurrentGroup));
			&MyDBMClose(*ArticleDB,&ArticleDBFilename($CurrentGroup));
			$CurrentGroupThreadDBLoaded = 0;
		}
		$CurrentGroup = "";
		$CurrentGroupLowWaterMark = 0;
		$CurrentGroupHighWaterMark = 0;
		
		# Close the NNTP connection and write server info to disk.
		&CloseTCPSocket(NNTPSERVER);
		if ($CurrentNNTPServerAllGroupsLoaded) {
			&MyDBMClose(*AllGroups,&AllGroupsFilename($CurrentNNTPServer));
			$CurrentNNTPServerAllGroupsLoaded = 0;
		}
		$CurrentNNTPServer = "";
	}
	if ($CurrentNNTPServer ne $NNTPServer) {		
		&SetCurrentGroup("",0);
		&CloseTCPSocket(NNTPSERVER) if ($CurrentNNTPServer ne "");
		if ($CurrentNNTPServerAllGroupsLoaded) {
			&MyDBMClose(*AllGroups,&AllGroupsFilename($CurrentNNTPServer));
			$CurrentNNTPServerAllGroupsLoaded = 0;
		}
		$CurrentNNTPServer = "";
		
		if ($LogChanges) {
			if ($NNTPServer eq "") {
				print STDERR "Clearing current news server.\n";
			} else {
				print STDERR "Changing to news server \"$NNTPServer\".\n";
			}
		}
		
		return 1 if ($NNTPServer eq "");		# ignore $LoadAllGroups if there's no server
		
		local($NNTPHost,$NNTPPort) = split(/\:/,$NNTPServer);
		$NNTPPort = 119 if ($NNTPPort eq "");
#		print STDERR "$NNTPHost : $NNTPPort\n";
		if (!&OpenActiveTCPSocket(NNTPSERVER,$NNTPHost,$NNTPPort)) {
			print STDERR "Couldn't open connection to news server $NNTPServer: $!\n";
			&Fail($ERR_GENERIC,"Could not open connection to news server \"$NNTPServer\": ".$!);
		}
		local($ResponseCode,$ResponseText) = &ReadNNTPResponseLine();
		if ($ResponseCode == 200) {
			$PostingAllowed = 1;
		} elsif ($ResponseCode == 201) {
			$PostingAllowed = 0;
		} else {
			print STDERR "NNTP server returned non-welcoming welcome message: $ResponseCode $ResponseText";
			&CloseTCPSocket(NNTPSERVER);
			&Fail($ResponseCode,$ResponseText);
		}
		# Not all NNTP servers understand MODE READER, but some of them require it.
		# So we send it no matter what, and ignore the response code.
		&SendNNTPCommand("MODE READER\015\012");
		$CurrentNNTPServer = $NNTPServer;
		$NNTPConnectionOpenedAt = time;
	}
	
	if ($LoadAllGroups) {
		if (!$CurrentNNTPServerAllGroupsLoaded) {
			if (!&MyDBMOpen(*AllGroups,&AllGroupsFilename($CurrentNNTPServer))) {
				$AllGroups{"","version"} = $CurrentAllGroupsVersion;
			} else {
				&UpdateAllGroupsVersion();
			}
			$CurrentNNTPServerAllGroupsLoaded = 1;
		}
		&UpdateAllGroupsTree();
	}
	
	1;
}

# SetCurrentGroup($GroupName,$LoadThreadDB)
# Change $CurrentGroup to the given one. If $LoadThreadDB is 1,
# then load and update $ThreadDB, the threading information; otherwise leave it empty.

sub SetCurrentGroup {
	local($GroupName,$LoadThreadDB) = @_;
	
	if ($GroupFilterPattern ne "" && $GroupName ne "" && $GroupName =~ /$GroupFilterPattern/o) {
		# Fail if the group is not allowed. $GroupFilterPattern is set in "config.pl".
		&Fail($ERR_GENERIC,"Sorry, you are not allowed to read the group $GroupName");
	}
	
	if ($CurrentGroup ne $GroupName) {
		if ($CurrentGroupThreadDBLoaded) {
			&MyDBMClose(*ThreadDB,&ThreadDBFilename($CurrentGroup));
			&MyDBMClose(*ArticleDB,&ArticleDBFilename($CurrentGroup));
			$CurrentGroupThreadDBLoaded = 0;
		}
		$CurrentGroup = "";
		$CurrentGroupLowWaterMark = 0;
		$CurrentGroupHighWaterMark = 0;
		if ($LogChanges) {
			if ($GroupName eq "") {
				print STDERR "Clearing current newsgroup.\n";
			} else {
				print STDERR "Changing newsgroup to \"$GroupName\".\n";
			}
		}
	}
	
	return 1 if ($GroupName eq "");
	
	# We set the group again, even if it's already the current group,
	# in order to find out the current low and high water marks.
	
	local($ResponseCode,$ResponseText) = &SendNNTPCommand("GROUP $GroupName\012");
	if ($ResponseCode != 211) {
		if ($ResponseCode == 411) {
			print STDERR "In SetCurrentGroup: group $GroupName does not exist on server $CurrentNNTPServer\n";
		} else {
			print STDERR "In SetCurrentGroup: got unexpected response from NNTP server when trying to set group: $ResponseCode $ResponseText\n";
		}
		&Fail($ResponseCode,$ResponseText);
	} else {
		if ($ResponseText =~ /^\d+\s+(\d+)\s+(\d+)/) {
			$CurrentGroupLowWaterMark = $1;
			$CurrentGroupHighWaterMark = $2;
		} else {
			print STDERR "In SetCurrentGroup: weird response with code 211: $ResponseText\n";
			&Fail($ResponseCode,$ResponseText);
		}
	}
	
	$CurrentGroup = $GroupName;
	
	if ($LoadThreadDB) {
		if (!$CurrentGroupThreadDBLoaded) {
			if (!&MyDBMOpen(*ThreadDB,&ThreadDBFilename($CurrentGroup))) {
				$ThreadDB{"","version"} = $CurrentThreadDBVersion;
				$ThreadDB{"DBRange","low"} = 1;
				$ThreadDB{"DBRange","high"} = 0;
			} else {
				&UpdateThreadDBVersion();
			}
			&MyDBMOpen(*ArticleDB,&ArticleDBFilename($CurrentGroup));
			$CurrentGroupThreadDBLoaded = 1;
		}
		&UpdateThreadDB();
	}
	
	1;
}

# SetCurrentUser($NewUser)
# Change $CurrentUser, the currently active user, to the given one.

sub SetCurrentUser {
	local($NewUser) = @_;
	
	return if ($NewUser eq $CurrentUser);
	
	&AddUserToCache($CurrentUser);
	
	if (&FetchUserFromCache($NewUser)) {
		if ($LogChanges) {
			print STDERR "Changing user to $NewUser.\n";
		}
		return;
	}
	
	# The new user isn't in the cache. Open their files.	
	&ClearSubscriptionCache();
	%Newsrc = ();
	%KillFile = ();
	
	if ($NewUser ne "") {
		if (!&MyDBMOpen(*Newsrc,&NewsrcFilename($NewUser))) {
			$Newsrc{"","version"} = $CurrentNewsrcVersion;
		} else {
			&UpdateNewsrcVersion();
		}
		if (!&MyDBMOpen(*KillFile,&KillFilename($NewUser))) {
			$KillFile{"","version"} = $CurrentKillFileVersion;
		} else {
			&UpdateKillFileVersion();
		}
		# Mark the account as last used "now".
		$Newsrc{"time","used"} = "".time;		# '"".' makes sure the time is a string
		if ($LogChanges) {
			print STDERR "Changing user to $NewUser.\n";
		}
	} else {
		if ($LogChanges) {
			print STDERR "Clearing current user.\n";
		}
	}
	$CurrentUser = $NewUser;
}

# CleanUpGlobalVars()
# Clean up and write out data files in preparation for shutting down.

sub CleanupGlobalVars {
	$RemoteHostName = "";
	$RemoteIPNum = "";
	&CloseTCPSocket(CLIENT);
	&CloseTCPSocket(LISTENER);
	print STDERR "Not listening for connections.\n";
	
	&SetCurrentGroup("",0);
	&SetCurrentNNTPServer("",0);
	&SetCurrentUser("");
	
	&FlushCaches();
}

# PipeSigHandler($sig)
# We got a SIGPIPE signal, meaning a broken pipe. Fail out; hopefully we'll
# end up back in the main loop without breaking anything.
# We may get a pipe signal when we try to send to CLIENT or NNTPSERVER and
# the connection is broken. Sometimes when the pipe is broken, depending on
# operating system and Larry Wall-knows what else, we get a SIGPIPE. Other
# times the print command simply fails.

sub PipeSigHandler {
	if ($NNTPCommandInProgress) {
		$NNTPCommandInProgress = 0;
		# For now, we're not making any attempt to recover.
		# In the worst case, recovery is pretty hard.
		&Fail($ERR_CLIENT_DIED,"The NNTP connection is broken.");
	} else {
		&Fail($ERR_CLIENT_DIED,"Probably, the client closed the network connection early. (1)");
	}
}

# ErrorBacktrace()
# Print information from the call stack.
# This is useful for locating errors.

sub ErrorBacktrace {
	local($package,$filename,$line,$subname,$hasargs,$wantarray);
	print $i;
	
	print "-- -- -- error backtrace:\n";
	$i = 1;
	while(($package,$filename,$line,$subname,$hasargs,$wantarray) = caller($i)) {
		print STDERR "$filename $line: $subname\n";
		++$i;
	}
	print "-- -- --\n";
}	


# IntHandler($sig)
# We got one or another interrupt signal. Try to shut down cleanly.

sub IntHandler {
	local($sig) = @_;
	
	&CleanupGlobalVars();

	print STDERR "Shut down in response to a SIG$sig.\n";

	exit(1);
}

# DefaultSigHandler($sig)
# We got an unexpected signal. Fall down and die.

sub DefaultSigHandler {
	local($sig) = @_;
	
	print STDERR "Received an unexpected SIG$sig. The server is shutting down,\n",
				 "not saving its data files because they might be corrupted.\n";
	&ErrorBacktrace();
	
	exit(1);
}

sub UpdateNewsrcVersion {
	local($NewsrcVersion) = $Newsrc{"","version"};
	
	# Now do something based on $NewsrcVersion and $CurrentNewsrcVersion.
	
	$Newsrc{"","version"} = $CurrentNewsrcVersion;
}

sub UpdateKillFileVersion {
	local($KillFileVersion) = $KillFile{"","version"};
	
	# Now do something based on $KillFileVersion and $CurrentKillFileVersion.
	
	$KillFile{"","version"} = $CurrentKillFileVersion;
}

sub UpdateThreadDBVersion {
	local($ThreadDBVersion) = $ThreadDB{"","version"};
	
	# Now do something based on $ThreadDBVersion and $CurrentThreadDBVersion.
	
	$ThreadDB{"","version"} = $CurrentThreadDBVersion;
}

sub UpdateAllGroupsVersion {
	local($AllGroupsVersion) = $AllGroups{"","version"};
	
	# Now do something based on $AllGroupsVersion and $CurrentAllGroupsVersion.
	
	$AllGroups{"","version"} = $CurrentAllGroupsVersion;
}

sub DoesUserExist {
	local($User) = @_;
	
	return 0 if ($User eq "");
	
	return -e $UserFilesDirectoryName."/".$User;
}

# NewsrcFilename($User)
# Return the pathname of the Newsrc file for the given user,
# creating its directory if necessary.

sub NewsrcFilename {
	local($User) = @_;
	
	return "" if ($User eq "");
	
	&MakeDirectory($UserFilesDirectoryName."/".$User,0700,"Couldn't make directory for user $User: $!");
	
	return $UserFilesDirectoryName."/".$User."/Newsrc";
}

# KillFileName($User)
# Return the pathname of the Kill file for the given user,
# creating its directory if necessary.

sub KillFilename {
	local($User) = @_;
	
	return "" if ($User eq "");
	
	&MakeDirectory($UserFilesDirectoryName."/".$User,0700,"Couldn't make directory for user $User: $!");
	
	return $UserFilesDirectoryName."/".$User."/Kill";
}

# ThreadDBFilename($Group)
# Return the pathname of the Thread.DB file for a given newsgroup,
# creating its directory if necessary.

sub ThreadDBFilename {
	local($Group) = @_;
	
	return "" if ($Group eq "");
	
	local(@GroupNameParts) = split(/\./,$Group);
	local($Server) = $CurrentNNTPServer;
	$Server =~ s/\:/\./;
	unshift(@GroupNameParts,$Server);
	local($DirectoryName) = $ThreadDBFilesDirectoryName."/".join('/',@GroupNameParts);
	
	&MakeDirectory($DirectoryName,0700,"Couldn't make ThreadDB directory for group \"$CurrentNNTPServer $Group\": $!");

	return $DirectoryName."/Thread.DB";
}

# ArticleDBFilename($Group)
# Return the pathname of the Article.DB file for a given newsgroup,
# creating its directory if necessary. This file is in the same
# directory as the Thread.DB file.

sub ArticleDBFilename {
	local($Group) = @_;
	
	return "" if ($Group eq "");
	
	local(@GroupNameParts) = split(/\./,$Group);
	local($Server) = $CurrentNNTPServer;
	$Server =~ s/\:/\./;
	unshift(@GroupNameParts,$Server);
	local($DirectoryName) = $ThreadDBFilesDirectoryName."/".join('/',@GroupNameParts);
	
	&MakeDirectory($DirectoryName,0700,"Couldn't make ArticleDB directory for group \"$CurrentNNTPServer $Group\": $!");

	return $DirectoryName."/Article.DB";
}

# AllGroupsFileName($Server)
# Return the pathname of the allgroups file for a given server.

sub AllGroupsFilename {
	local($Server) = @_;
	
	return "" if ($Server eq "");
	
	$Server =~ s/\:/\./;
	
	return $AllGroupsFilesDirectoryName."/".$Server;
}


# MakeDirectory($DirectoryName,$Mode,$FailureMessage)
# Create a directory from a pathname, and any intermediate directories on
# the path that don't already exist.

sub MakeDirectory {
	local($DirectoryName,$Mode,$FailureMessage) = @_;
	
	local($WorkingDirectory) = `pwd`;
	chop($WorkingDirectory);
	
#	print STDERR "In MakeDirectory: \$DirectoryName is \"$DirectoryName\"\n";
#	print STDERR "    \$Mode is \"$Mode\"\n";
	
	chdir("/") || &Fail($ERR_COULDNT_MKDIR,$FailureMessage." (1)");
	
	local(@DirectoryNameParts) = split(/\//,$DirectoryName);
	shift(@DirectoryNameParts); 	# since there was a leading slash
	local($DirectoryNamePart);
	foreach $DirectoryNamePart (@DirectoryNameParts) {
		if (!(-e $DirectoryNamePart) || !(-d $DirectoryNamePart)) {
			if (-e $DirectoryNamePart) {
				# Then it must not be a directory, which is not what we want.
				$! = "file of that name exists.";
				&Fail($ERR_COULDNT_MKDIR,$FailureMessage." (2)");
			}
			mkdir($DirectoryNamePart,$Mode) || &Fail($ERR_COULDNT_MKDIR,$FailureMessage." (3)");
		}
		chdir($DirectoryNamePart) || &Fail($ERR_COULDNT_MKDIR,$FailureMessage." (4)");
	}
	
	chdir($WorkingDirectory) || &Fail($ERR_COULDNT_MKDIR,$FailureMessage." (5)");
}


# ValidEmailAddress($Email)
# Return whether the given putative e-mail address $Email appears
# superficially valid. It is valid if either $EmailValidityPattern
# is "", meaning that no validity check is to be done, or if it matches
# $EmailValidityPattern as a regular expression. $EmailValidityPattern
# is set in "config.pl".

sub ValidEmailAddress {
	local($Email) = @_;
	
	return 1 if ($EmailValidityPattern eq "");
	return 0 if ($Email eq "");
	return $Email =~ /$EmailValidityPattern/o;
}

1;
